@@ -17,10 +17,7 @@ Copyright (c) 2015, 2020 Tim Waugh <tim@cyberelk.net>
 """
 
 from datetime import datetime
-from email.mime.text import MIMEText
-from email import charset
 from flexmock import flexmock
-from tests.util import Watcher
 from journal_brief.cli.constants import (EMAIL_SUPPRESS_EMPTY_TEXT,
                                          EMAIL_DRY_RUN_SEPARATOR)
 from journal_brief.cli.main import CLI
@@ -28,12 +25,10 @@ import json
 import logging
 import os
 import pytest
-import smtplib
-import ssl
-import subprocess
 from systemd import journal
 from tempfile import NamedTemporaryFile
 from tests.test_filter import MySpecialFormatter  # registers class; # noqa: F401
+from tests.util import Watcher
 import uuid
 import yaml
 
@@ -42,12 +37,19 @@ logging.basicConfig(level=logging.DEBUG)
 
 
 @pytest.fixture
-def config_and_cursor(tmp_path):
+def build_config_and_cursor(tmp_path):
     with NamedTemporaryFile(mode='wt', dir=tmp_path) as configfile:
         with NamedTemporaryFile(mode='w+t', dir=tmp_path) as cursorfile:
-            configfile.write('cursor-file: {0}\n'.format(cursorfile.name))
-            configfile.flush()
-            yield (configfile, cursorfile)
+            def write_config(config=None):
+                configfile.write('cursor-file: {0}\n'.format(cursorfile.name))
+                if isinstance(config, dict):
+                    configfile.write(yaml.dump(config))
+                elif config:
+                    configfile.write(config)
+                configfile.flush()
+                return (configfile, cursorfile)
+
+            yield write_config
 
 
 @pytest.fixture
@@ -78,7 +80,7 @@ class TestCLI(object):
                             '-p', 'debug'])
             assert cli.config.get('priority') == 'debug'
 
-    def test_normal_run(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_normal_run(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         (flexmock(journal.Reader, add_match=None, add_disjunction=None)
             .should_receive('get_next')
             .and_return({'__CURSOR': '1',
@@ -89,7 +91,7 @@ class TestCLI(object):
                          'MESSAGE': 'message2'})
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
@@ -97,7 +99,7 @@ class TestCLI(object):
         assert not err
         assert len(out.splitlines()) == 2
 
-    def test_dry_run(self, config_and_cursor):
+    def test_dry_run(self, build_config_and_cursor):
         (flexmock(journal.Reader, add_match=None, add_disjunction=None)
             .should_receive('get_next')
             .and_return({'__CURSOR': '1',
@@ -105,12 +107,12 @@ class TestCLI(object):
                          'MESSAGE': 'message'})
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cli = CLI(args=['--conf', configfile.name, '--dry-run'])
         cli.run()
         assert not cursorfile.read()
 
-    def test_this_boot(self, config_and_cursor):
+    def test_this_boot(self, build_config_and_cursor):
         final_cursor = '1'
         flexmock(journal.Reader, add_match=None, add_disjunction=None)
         (flexmock(journal.Reader)
@@ -123,7 +125,7 @@ class TestCLI(object):
                          'MESSAGE': 'message'})
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cursorfile.write(final_cursor)
         cursorfile.flush()
         cli = CLI(args=['--conf', configfile.name, '-b'])
@@ -131,7 +133,7 @@ class TestCLI(object):
         cursorfile.seek(0)
         assert cursorfile.read() == final_cursor
 
-    def test_log_level(self, config_and_cursor, missing_or_empty_cursor):
+    def test_log_level(self, build_config_and_cursor, missing_or_empty_cursor):
         flexmock(journal.Reader, add_match=None, add_disjunction=None)
         (flexmock(journal.Reader)
             .should_receive('log_level')
@@ -141,12 +143,12 @@ class TestCLI(object):
             .should_receive('get_next')
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cli = CLI(args=['--conf', configfile.name, '-p', 'err'])
         cli.run()
 
-    def test_reset(self, config_and_cursor):
-        (configfile, cursorfile) = config_and_cursor
+    def test_reset(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor()
         with cursorfile:  # force the cursorfile context to be exited at the right time
             cli = CLI(args=['--conf', configfile.name, 'reset'])
             cli.run()
@@ -159,7 +161,7 @@ class TestCLI(object):
         cli.run()
         assert not os.access(cursorfile.name, os.F_OK)
 
-    def test_stats(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_stats(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         (flexmock(journal.Reader, add_match=None, add_disjunction=None)
             .should_receive('get_next')
             .and_return({'__CURSOR': '1',
@@ -170,12 +172,10 @@ class TestCLI(object):
                          'MESSAGE': 'include'})
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write("""
+        (configfile, cursorfile) = build_config_and_cursor("""
 exclusions:
 - MESSAGE: [exclude]
 """)
-        configfile.flush()
         cli = CLI(args=['--conf', configfile.name, 'stats'])
         cli.run()
 
@@ -185,7 +185,7 @@ exclusions:
                                  "         1  {'MESSAGE': ['exclude']}",
                                  ""])
 
-    def test_debrief(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_debrief(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         entries = [
             {'__CURSOR': '1',
              'MESSAGE': 'message 1',
@@ -216,7 +216,7 @@ exclusions:
 
         expectation.and_return({})
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cli = CLI(args=['--conf', configfile.name, 'debrief'])
         cli.run()
 
@@ -232,7 +232,7 @@ exclusions:
             "    - message 2",
             ''])
 
-    def test_debrief_no_input(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_debrief_no_input(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         """
         Check it handles there being no input
         """
@@ -240,7 +240,7 @@ exclusions:
             .should_receive('get_next')
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cli = CLI(args=['--conf', configfile.name, 'debrief'])
         cli.run()
 
@@ -248,7 +248,7 @@ exclusions:
         assert not err
         assert not out
 
-    def test_exclusions_yaml(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_exclusions_yaml(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         (flexmock(journal.Reader, add_match=None, add_disjunction=None)
             .should_receive('get_next')
             .and_return({'__CURSOR': '1',
@@ -256,12 +256,10 @@ exclusions:
                          'MESSAGE': 'message'})
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write("""
+        (configfile, cursorfile) = build_config_and_cursor("""
 exclusions:
 - MESSAGE: [1]
 """)
-        configfile.flush()
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
@@ -269,7 +267,7 @@ exclusions:
         assert not err
         assert 'message' in out
 
-    def test_inclusions_yaml(self, config_and_cursor, missing_or_empty_cursor):
+    def test_inclusions_yaml(self, build_config_and_cursor, missing_or_empty_cursor):
         (flexmock(journal.Reader)
             .should_receive('get_next')
             .and_return({}))
@@ -284,14 +282,12 @@ exclusions:
             .should_receive('add_disjunction')
             .replace_with(watcher.watch_call('add_disjunction')))
 
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write("""
+        (configfile, cursorfile) = build_config_and_cursor("""
 inclusions:
 - PRIORITY: [0, 1, 2, 3]
 - PRIORITY: [4, 5, 6]
   _SYSTEMD_UNIT: [myservice.service]
 """)
-        configfile.flush()
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
@@ -317,7 +313,7 @@ inclusions:
         # And nothing else
         assert len(watcher.calls) == 9
 
-    def test_multiple_output_formats_cli(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_multiple_output_formats_cli(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         entry = {
             '__CURSOR': '1',
             '__REALTIME_TIMESTAMP': datetime.now(),
@@ -329,7 +325,7 @@ inclusions:
             .and_return(entry)
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cli = CLI(args=['--conf', configfile.name,
                         '-o', 'cat,cat,json'])
         cli.run()
@@ -344,7 +340,7 @@ inclusions:
         del output['__REALTIME_TIMESTAMP']
         assert output == entry
 
-    def test_multiple_output_formats_conf(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_multiple_output_formats_conf(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         entry = {
             '__CURSOR': '1',
             '__REALTIME_TIMESTAMP': datetime.now(),
@@ -356,14 +352,12 @@ inclusions:
             .and_return(entry)
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write("""
+        (configfile, cursorfile) = build_config_and_cursor("""
 output:
 - cat
 - cat
 - json
 """)
-        configfile.flush()
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
@@ -377,7 +371,7 @@ output:
         del output['__REALTIME_TIMESTAMP']
         assert output == entry
 
-    def test_formatter_filter(self, capsys, config_and_cursor, missing_or_empty_cursor):
+    def test_formatter_filter(self, capsys, build_config_and_cursor, missing_or_empty_cursor):
         """
         Just a coverage test
         """
@@ -396,7 +390,7 @@ output:
             .and_return(entry)
             .and_return({}))
 
-        (configfile, cursorfile) = config_and_cursor
+        (configfile, cursorfile) = build_config_and_cursor()
         cli = CLI(args=['--conf', configfile.name,
                         '-p', 'err', '-o', 'login'])
         cli.run()
@@ -414,123 +408,256 @@ output:
         assert out
 
 
-class TestCLIEmailCommand(object):
+class TestCLIEmailBase(object):
+    @pytest.fixture(autouse=True)
+    def mock_journal(self, mocker):
+        # mock SelectiveReader to ensure that systemd.journal.Reader is never used
+        self.reader_class = mocker.patch('journal_brief.cli.main.SelectiveReader', autospec=True)
+
+        self.entries_class = mocker.patch('journal_brief.cli.main.LatestJournalEntries', autospec=True)
+        self.entries_object = self.entries_class.return_value.__enter__.return_value
+        # don't provide any journal entries unless test function calls mock_entries()
+        self.entries_object.__iter__.return_value = []
+
+        def mock_entries(entries):
+            self.entries_object.__iter__.return_value = entries
+
+        yield mock_entries
+
+
+class TestCLIEmailMIME(TestCLIEmailBase):
     TEST_COMMAND = 'foo'
+    TEST_SUBJECT = 'subj'
 
-    def test(self, config_and_cursor, missing_or_empty_cursor):
-        entry = {
-            '__CURSOR': '1',
-            'TEST': 'yes',
-            'OUTPUT': 'message',
-        }
+    @pytest.fixture(autouse=True)
+    def mock_mime(self, mocker):
+        self.mimetext_class = mocker.patch('journal_brief.cli.main.MIMEText', autospec=True)
+        self.mimetext_object = self.mimetext_class.return_value
 
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return(entry)
-         .and_return({}))
+    @pytest.fixture(autouse=True)
+    def mock_subprocess(self, mocker):
+        self.subprocess_module = mocker.patch('journal_brief.cli.main.subprocess', autospec=True)
 
-        (flexmock(subprocess)
-         .should_receive('run')
-         .with_args(self.TEST_COMMAND, shell=True, check=True, text=True,
-                    input=entry['OUTPUT'])
-         .once())
+    def test(self, capsys, mocker, build_config_and_cursor, mock_journal):
+        entries = [
+            {
+                '__CURSOR': '1',
+                'TEST': 'yes',
+                'OUTPUT': 'message',
+            }
+        ]
 
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+        mock_journal(entries)
+
+        (configfile, cursorfile) = build_config_and_cursor({
             'output': 'test',
             'email': {
                 'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_dry_run(self, capsys, config_and_cursor):
-        entry = {
-            '__CURSOR': '1',
-            'TEST': 'yes',
-            'OUTPUT': 'message',
-        }
+        self.mimetext_class.assert_called_once_with(entries[0]['OUTPUT'], _charset=mocker.ANY)
 
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return(entry)
-         .and_return({}))
+        self.mimetext_object.__setitem__.assert_any_call('From', 'F')
+        self.mimetext_object.__setitem__.assert_any_call('To', 'T')
 
-        (flexmock(subprocess)
-         .should_receive('run')
-         .never())
+    def test_allow_empty(self, mocker, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
+            'email': {
+                'suppress_empty': False,
+                'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
+            },
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
 
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+        self.mimetext_class.assert_called_once_with(EMAIL_SUPPRESS_EMPTY_TEXT, _charset=mocker.ANY)
+
+    def test_subject(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
+            'email': {
+                'suppress_empty': False,
+                'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
+                'subject': self.TEST_SUBJECT,
+            },
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
+
+        self.mimetext_object.__setitem__.assert_any_call('Subject', self.TEST_SUBJECT)
+
+    def test_to_list(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
+            'email': {
+                'suppress_empty': False,
+                'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': ['A', 'B'],
+            },
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
+
+        self.mimetext_object.__setitem__.assert_any_call('To', 'A, B')
+
+    def test_cc_list(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
+            'email': {
+                'suppress_empty': False,
+                'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
+                'cc': ['A', 'B'],
+            },
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
+
+        self.mimetext_object.__setitem__.assert_any_call('Cc', 'A, B')
+
+    def test_bcc_list(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
+            'email': {
+                'suppress_empty': False,
+                'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
+                'bcc': ['A', 'B'],
+            },
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
+
+        self.mimetext_object.__setitem__.assert_any_call('Bcc', 'A, B')
+
+    def test_headers(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
+            'email': {
+                'suppress_empty': False,
+                'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
+                'headers': {
+                    'X-Header-1': '1',
+                    'X-Header-4': '4',
+                },
+            },
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
+
+        self.mimetext_object.__setitem__.assert_any_call('X-Header-1', '1')
+        self.mimetext_object.__setitem__.assert_any_call('X-Header-4', '4')
+
+
+class TestCLIEmailCommand(TestCLIEmailBase):
+    TEST_COMMAND = 'foo'
+
+    @pytest.fixture(autouse=True)
+    def mock_subprocess(self, mocker):
+        self.subprocess_module = mocker.patch('journal_brief.cli.main.subprocess', autospec=True)
+
+    def test(self, build_config_and_cursor, mock_journal, mocker):
+        entries = [
+            {
+                '__CURSOR': '1',
+                'TEST': 'yes',
+                'OUTPUT': 'message',
+            }
+        ]
+
+        mock_journal(entries)
+
+        (configfile, cursorfile) = build_config_and_cursor({
             'output': 'test',
             'email': {
                 'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
+            }
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
+
+        self.subprocess_module.run.assert_called_once_with(self.TEST_COMMAND, shell=True, check=True, text=True, input=mocker.ANY)
+        assert isinstance(self.subprocess_module.run.call_args[1]['input'], str), "subprocess.run 'input' argument must be a string"
+
+    def test_non_mime(self, build_config_and_cursor, mock_journal, mocker):
+        entries = [
+            {
+                '__CURSOR': '1',
+                'TEST': 'yes',
+                'OUTPUT': 'message',
+            }
+        ]
+
+        mock_journal(entries)
+
+        (configfile, cursorfile) = build_config_and_cursor({
+            'output': 'test',
+            'email': {
+                'command': self.TEST_COMMAND,
+            }
+        })
+        cli = CLI(args=['--conf', configfile.name])
+        cli.run()
+
+        self.subprocess_module.run.assert_called_once_with(self.TEST_COMMAND, shell=True, check=True, text=True, input=entries[0]['OUTPUT'])
+
+    def test_dry_run(self, capsys, build_config_and_cursor, mock_journal):
+        entries = [
+            {
+                '__CURSOR': '1',
+                'TEST': 'yes',
+                'OUTPUT': 'message',
+            }
+        ]
+
+        mock_journal(entries)
+
+        (configfile, cursorfile) = build_config_and_cursor({
+            'output': 'test',
+            'email': {
+                'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--dry-run', '--conf', configfile.name])
         cli.run()
 
         (out, err) = capsys.readouterr()
         assert not err
         lines = out.splitlines()
-        assert len(lines) == 3
+        assert len(lines) == 10
         assert lines[0] == "Email to be delivered via '{0}'".format(self.TEST_COMMAND)
         assert lines[1] == EMAIL_DRY_RUN_SEPARATOR
 
-    def test_allow_empty(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
+        self.subprocess_module.run.assert_not_called()
 
-        (flexmock(subprocess)
-         .should_receive('run')
-         .with_args(self.TEST_COMMAND, shell=True, check=True, text=True,
-                    input=EMAIL_SUPPRESS_EMPTY_TEXT)
-         .once())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+    def test_suppress_empty(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
             'email': {
-                'suppress_empty': False,
                 'command': self.TEST_COMMAND,
+                'from': 'F',
+                'to': 'T',
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_suppress_empty(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
-
-        (flexmock(subprocess)
-         .should_receive('run')
-         .never())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
-            'email': {
-                'command': self.TEST_COMMAND,
-            },
-        }))
-        configfile.flush()
-        cli = CLI(args=['--conf', configfile.name])
-        cli.run()
+        self.subprocess_module.run.assert_not_called()
 
 
-class EmailMessageMatcher(object):
-    def __init__(self, msg):
-        self.msg = msg
-
-    def __eq__(self, msg):
-        return str(self.msg) == str(msg)
-
-
-class TestCLIEmailSMTP(object):
+class TestCLIEmailSMTP(TestCLIEmailBase):
     TEST_USER = 'zork'
     TEST_PASSWORD = 'xyzzy'
     TEST_HOST = 'example'
@@ -538,322 +665,160 @@ class TestCLIEmailSMTP(object):
     TEST_SUBJECT = 'subj'
 
     @pytest.fixture(autouse=True)
-    def smtp_mock_fixer(self):
-        # ensure that no attempt is made to make a connection
-        (flexmock(smtplib.SMTP)
-         .should_receive('connect')
-         .with_args(str, int))
-        # in Python versions before 3.8, the `close` method will fail if
-        # no connection was opened; because the object has been mocked,
-        # that will always be the case
-        (flexmock(smtplib.SMTP)
-         .should_receive('close')
-         .and_return(None))
+    def mock_smtp(self, mocker):
+        self.smtp_class = mocker.patch('journal_brief.cli.main.SMTP', autospec=True)
+        self.smtp_object = self.smtp_class.return_value
+        self.smtp_context = self.smtp_object.__enter__.return_value
 
-    def test(self, capsys, config_and_cursor, missing_or_empty_cursor):
-        entry = {
-            '__CURSOR': '1',
-            'TEST': 'yes',
-            'OUTPUT': 'message',
-        }
+    def test(self, capsys, mocker, build_config_and_cursor, mock_journal):
+        entries = [
+            {
+                '__CURSOR': '1',
+                'TEST': 'yes',
+                'OUTPUT': 'message',
+            }
+        ]
 
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return(entry)
-         .and_return({}))
+        mock_journal(entries)
 
-        charset.add_charset('utf-8', charset.QP, charset.QP)
-        message = MIMEText(entry['OUTPUT'], _charset='utf-8')
-        message['From'] = 'F'
-        message['To'] = 'T'
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .with_args(EmailMessageMatcher(message))
-         .once())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+        (configfile, cursorfile) = build_config_and_cursor({
             'output': 'test',
             'email': {
-                'smtp': {
-                    'from': 'F',
-                    'to': 'T',
-                },
+                'from': 'F',
+                'to': 'T',
+                'smtp': {},
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_dry_run(self, capsys, config_and_cursor):
-        entry = {
-            '__CURSOR': '1',
-            'TEST': 'yes',
-            'OUTPUT': 'message',
-        }
+        self.smtp_context.send_message.assert_called_once()
 
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return(entry)
-         .and_return({}))
+    def test_dry_run(self, capsys, build_config_and_cursor, mock_journal):
+        entries = [
+            {
+                '__CURSOR': '1',
+                'TEST': 'yes',
+                'OUTPUT': 'message',
+            }
+        ]
 
-        (flexmock(smtplib)
-         .should_receive('SMTP')
-         .never())
+        mock_journal(entries)
 
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+        (configfile, cursorfile) = build_config_and_cursor({
             'output': 'test',
             'email': {
-                'smtp': {
-                    'from': 'F',
-                    'to': 'T',
-                },
+                'from': 'F',
+                'to': 'T',
+                'smtp': {},
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--dry-run', '--conf', configfile.name])
         cli.run()
 
+        self.smtp_class.assert_not_called()
+
         (out, err) = capsys.readouterr()
         assert not err
         lines = out.splitlines()
-        assert len(lines) == 9
+        assert len(lines) == 10
         assert lines[0] == 'Email to be delivered via SMTP to localhost port 25'
         assert lines[1] == EMAIL_DRY_RUN_SEPARATOR
 
-    def test_allow_empty(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
-
-        charset.add_charset('utf-8', charset.QP, charset.QP)
-        message = MIMEText(EMAIL_SUPPRESS_EMPTY_TEXT, _charset='utf-8')
-        message['From'] = 'F'
-        message['To'] = 'T'
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .with_args(EmailMessageMatcher(message))
-         .once())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+    def test_suppress_empty(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
             'email': {
-                'suppress_empty': False,
-                'smtp': {
-                    'from': 'F',
-                    'to': 'T',
-                },
+                'from': 'F',
+                'to': 'T',
+                'smtp': {},
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_suppress_empty(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
+        self.smtp_class.assert_not_called()
 
-        (flexmock(smtplib)
-         .should_receive('SMTP')
-         .never())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
-            'email': {
-                'smtp': {
-                    'from': 'F',
-                    'to': 'T',
-                },
-            },
-        }))
-        configfile.flush()
-        cli = CLI(args=['--conf', configfile.name])
-        cli.run()
-
-    def test_host(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('__init__')
-         .with_args(self.TEST_HOST, 0)
-         .once())
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .once())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+    def test_host(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
             'email': {
                 'suppress_empty': False,
+                'from': 'F',
+                'to': 'T',
                 'smtp': {
                     'host': self.TEST_HOST,
-                    'from': 'F',
-                    'to': 'T',
                 },
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_port(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
+        self.smtp_class.assert_called_once_with(self.TEST_HOST, 0)
 
-        (flexmock(smtplib.SMTP)
-         .should_receive('__init__')
-         .with_args(self.TEST_HOST, self.TEST_PORT)
-         .once())
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .once())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+    def test_port(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
             'email': {
                 'suppress_empty': False,
+                'from': 'F',
+                'to': 'T',
                 'smtp': {
                     'host': self.TEST_HOST,
                     'port': self.TEST_PORT,
-                    'from': 'F',
-                    'to': 'T',
                 },
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_starttls(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
+        self.smtp_class.assert_called_once_with(self.TEST_HOST, self.TEST_PORT)
 
-        (flexmock(smtplib.SMTP)
-         .should_receive('starttls')
-         .with_args(context=ssl.SSLContext)
-         .once()
-         .ordered())
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .once()
-         .ordered())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+    def test_starttls(self, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
             'email': {
                 'suppress_empty': False,
+                'from': 'F',
+                'to': 'T',
                 'smtp': {
                     'starttls': True,
-                    'from': 'F',
-                    'to': 'T',
                 },
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_user(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
+        assert 'starttls' == self.smtp_context.method_calls[0][0]
+        assert 'send_message' == self.smtp_context.method_calls[1][0]
 
-        (flexmock(smtplib.SMTP)
-         .should_receive('login')
-         .with_args(self.TEST_USER, None)
-         .once()
-         .ordered())
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .once()
-         .ordered())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+    def test_user(self, mocker, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
             'email': {
                 'suppress_empty': False,
+                'from': 'F',
+                'to': 'T',
                 'smtp': {
                     'user': self.TEST_USER,
-                    'from': 'F',
-                    'to': 'T',
                 },
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_password(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
+        assert mocker.call.login(self.TEST_USER, None) == self.smtp_context.method_calls[0]
+        assert 'send_message' == self.smtp_context.method_calls[1][0]
 
-        (flexmock(smtplib.SMTP)
-         .should_receive('login')
-         .with_args(self.TEST_USER, self.TEST_PASSWORD)
-         .once()
-         .ordered())
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .once()
-         .ordered())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
+    def test_password(self, mocker, build_config_and_cursor):
+        (configfile, cursorfile) = build_config_and_cursor({
             'email': {
                 'suppress_empty': False,
+                'from': 'F',
+                'to': 'T',
                 'smtp': {
                     'user': self.TEST_USER,
                     'password': self.TEST_PASSWORD,
-                    'from': 'F',
-                    'to': 'T',
                 },
             },
-        }))
-        configfile.flush()
+        })
         cli = CLI(args=['--conf', configfile.name])
         cli.run()
 
-    def test_subject(self, config_and_cursor, missing_or_empty_cursor):
-        (flexmock(journal.Reader, add_match=None, add_disjunction=None)
-         .should_receive('get_next')
-         .and_return({}))
-
-        charset.add_charset('utf-8', charset.QP, charset.QP)
-        message = MIMEText(EMAIL_SUPPRESS_EMPTY_TEXT, _charset='utf-8')
-        message['From'] = 'F'
-        message['To'] = 'T'
-        message['Subject'] = self.TEST_SUBJECT
-
-        (flexmock(smtplib.SMTP)
-         .should_receive('send_message')
-         .with_args(EmailMessageMatcher(message))
-         .once())
-
-        (configfile, cursorfile) = config_and_cursor
-        configfile.write(yaml.dump({
-            'email': {
-                'suppress_empty': False,
-                'smtp': {
-                    'from': 'F',
-                    'to': 'T',
-                    'subject': self.TEST_SUBJECT,
-                },
-            },
-        }))
-        configfile.flush()
-        cli = CLI(args=['--conf', configfile.name])
-        cli.run()
+        assert mocker.call.login(self.TEST_USER, self.TEST_PASSWORD) == self.smtp_context.method_calls[0]
+        assert 'send_message' == self.smtp_context.method_calls[1][0]
